#ifndef _CAN_DRIVER_API_H
#define _CAN_DRIVER_API_H


#define CANDEV                      0xA460
#define CAN_SIMPLE_DEVICE_VERSION   0
#define CAN_DEVTYPE_NONE            0

/////////////////////////////////////////////////////////////////////////////
// I/O extended codes

#define CTL_CODE(DeviceType, Function, Method, Access) (                 \
    ((DeviceType) << 16) | ((Access) << 14) | ((Function) << 2) | (Method) )

#define METHOD_BUFFERED                 0
#define METHOD_IN_DIRECT                1
#define METHOD_OUT_DIRECT               2
#define METHOD_NEITHER                  3

#define FILE_ANY_ACCESS                 0
#define FILE_READ_ACCESS                ( 0x0001 )    // file & pipe
#define FILE_WRITE_ACCESS               ( 0x0002 )    // file & pipe


#define IOCTL_CANDEV_GET_DEVICE_VER \
  CTL_CODE(CANDEV, 0x900, METHOD_BUFFERED, FILE_READ_ACCESS)

#define IOCTL_CANDEV_INT_FLAGS \
  CTL_CODE(CANDEV, 0x901, METHOD_BUFFERED, FILE_READ_ACCESS)
  
#define IOCTL_CANDEV_GET_NBLOCK_RAW_DMA \
  CTL_CODE(CANDEV, 0x903, METHOD_BUFFERED, FILE_READ_ACCESS)
 
#define IOCTL_CANDEV_GET_STATUS \
  CTL_CODE(CANDEV, 0x904, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_CANDEV_READ_DMA_DATA_BLOCKS \
  CTL_CODE(CANDEV, 0x905, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_CANDEV_DISABLE_CANONICAL_DMA \
  CTL_CODE(CANDEV, 0x907, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_CANDEV_READ_DMA_CH_DATA_BLOCKS \
  CTL_CODE(CANDEV, 0x908, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_CANDEV_ENABLE_CANONICAL_DMA \
  CTL_CODE(CANDEV, 0x909, METHOD_BUFFERED, FILE_ANY_ACCESS)
    
#define IOCTL_CANDEV_GET_RESOURCE_INFO \
  CTL_CODE(CANDEV, 0x950, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_CANDEV_CLEAR_DMA \
  CTL_CODE(CANDEV, 0x953, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_CANDEV_READ_BAR \
  CTL_CODE(CANDEV, 0x954, METHOD_BUFFERED, FILE_READ_ACCESS)

#define IOCTL_CANDEV_WRITE_BAR \
  CTL_CODE(CANDEV, 0x955, METHOD_BUFFERED, FILE_WRITE_ACCESS)
  
#define IOCTL_CANDEV_DMA_COUNT \
  CTL_CODE(CANDEV, 0x966, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_CANDEV_READ_DMA_DATABUF \
  CTL_CODE(CANDEV, 0x967, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_CANDEV_READ_DMA_STATUS \
  CTL_CODE(CANDEV, 0x968, METHOD_BUFFERED, FILE_ANY_ACCESS)  

#define IOCTL_CANDEV_GET_PROPERTY \
  CTL_CODE(CANDEV, 0x96A, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_CANDEV_READ_DATABUF \
  CTL_CODE(CANDEV, 0x96C, METHOD_OUT_DIRECT, FILE_READ_ACCESS)

#define IOCTL_CANDEV_WRITE_DATABUF \
  CTL_CODE(CANDEV, 0x96D, METHOD_BUFFERED, FILE_WRITE_ACCESS)
  
#define IOCTL_CANDEV_WRITE_REG  \
  CTL_CODE(CANDEV, 0x97D, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_CANDEV_READ_REG    \
  CTL_CODE(CANDEV, 0x97E, METHOD_BUFFERED, FILE_ANY_ACCESS)
 

/////////////////////////////////////////////////////////////////////////////
//Structs

//IOCTL_CANDEV_GET_DEVICE_VER
typedef struct _CAN_GET_REV_OUT{
  UINT32 m_nRev;
} CAN_GET_REV_OUT;

//IOCTL_CANDEV_GET_NBLOCK_RAW_DMA
typedef struct _CAN_GET_NBLOCK_DMA_OUT{
  UINT32 m_nValue;
} CAN_GET_NBLOCK_DMA_OUT;

// IOCTL_CANDEV_READ_DMA_DATA_BLOCKS
typedef struct _CAN_READ_DMA_DATA_BLOCKS_IN{
  UINT32 m_nBlocks;        // Size (in bytes)
} CAN_READ_DMA_DATA_BLOCKS_IN;

// IOCTL_CANDEV_READ_DMA_CH_DATA_BLOCKS
typedef struct _CAN_READ_DMA_CH_DATA_BLOCKS_IN{
  UINT32 m_nBlocks;        // Size (in bytes)
  UINT32 m_nCh;            // Number of channel which should be read 
} CAN_READ_DMA_CH_DATA_BLOCKS_IN;

//IOCTL_CANDEV_GET_STATUS
typedef struct _CAN_GET_STATUS_IN{
  UINT32 m_nTime;
} CAN_GET_STATUS_IN;

// IOCTL_CAN_WRITE_REG
typedef struct _CAN_WRITE_REG_IN{
  UINT32 m_nAddress;
  UINT32 m_nData;
} CAN_WRITE_REG_IN;

typedef struct _CAN_WRITE_REG_OUT{
  UINT32 m_nData;
} CAN_WRITE_REG_OUT;

//IOCTL_CAN_READ_REG
typedef struct _CAN_READ_REG_IN{
  UINT32 m_nAddress;
} CAN_READ_REG_IN;

typedef struct _CAN_READ_REG_OUT{
  UINT32 m_nData;
} CAN_READ_REG_OUT;

//IOCTL_CANDEV_DMA_COUNT
typedef struct _CAN_DMA_COUNT_OUT{
  UINT32 m_nData;
} CAN_DMA_COUNT_OUT;

//IOCTL_CANDEV_READ_DMA_STATUS
typedef struct _CAN_READ_DMA_STATUS_OUT{
  UINT32 m_nStatus;
} CAN_READ_DMA_STATUS_OUT;

//IOCTL_CANDEV_INT_FLAGS
typedef struct _CAN_INT_FLAGS_OUT{
  UINT32 m_nMainInt;
  UINT32 m_nCan1Int;
  UINT32 m_nCan2Int;
} CAN_INT_FLAGS_OUT;

// IOCTL_CAN_GET_RESOURCE_INFO
typedef struct _CAN_GET_RESOURCE_INFO_OUT{
  UINT32 m_nBus;
  UINT32 m_nSlot;
  UINT32 m_nFunction;
  UINT32 m_nDataBufPhysAddr;
  UINT32 m_nDataBufSize;
  UINT32 m_BarPhysAddr;
  UINT8  m_szCompileDate[128];
} CAN_GET_RESOURCE_INFO_OUT;

// IOCTL_CAN_READ_DATABUF
typedef struct _CAN_READ_DATABUF_IN{
  UINT32 m_nOffset;      // Offset of data (in bytes)
  UINT32 m_nSize;        // Size (in bytes)
} CAN_READ_DATABUF_IN;

// IOCTL_CANDEV_WRITE_DATABUF
typedef struct _CAN_WRITE_DATABUF_IN{
  UINT32 m_nOffset;      // Offset of data (in bytes)
  UINT32 m_nSize;        // Size (in bytes)
  UINT8 m_Data[0];       // array of bytes
} CAN_WRITE_DATABUF_IN;

// IOCTL_CAN_READ_BAR
typedef struct _CAN_READ_BAR_IN{
  UINT32 m_nBar;
  UINT32 m_nOffset;      // Offset of data (in bytes)
  UINT32 m_nSize;        // Size (in bytes)
} CAN_READ_BAR_IN;

// IOCTL_CAN_WRITE_BAR
typedef struct _CAN_WRITE_BAR_IN{
  UINT32 m_nBar;
  UINT32 m_nOffset;      // Offset of data (in bytes)
  UINT32 m_nSize;        // Size (in bytes)
  UINT8 m_Data[0];       // array of bytes
} CAN_WRITE_BAR_IN;

/////////////////////////////////////////////////////////////////////////////
// Macros
#define GetAbsoluteDecimation(ch, pch, out){                                                                               \
  if((ch == 0)                                                                  \
     || (pch[ch].m_nDdcDataSource == CAN_DDC_DATA_SOURCE_REALTIME)            \
     || (pch[0].m_nDdcCicDecimation == 0))                                      \
    out = pch[ch].m_nDdcCicDecimation;                                          \
  else                                                                          \
    out = pch[0].m_nDdcCicDecimation * pch[ch].m_nDdcCicDecimation;             \
  if(out == 0)                                                                  \
    out = 1;                                                                    \
}
#endif
