
#ifndef _CAN_DEVICE_EX_H
#define _CAN_DEVICE_EX_H

#include "CANDevice.h"

/////////////////////////////////////////////////////////////////////////////
// Class CCANDeviceEx

class CCANDeviceEx : public CCANDevice{
//CRITICAL_SECTION for threadSafe
private:
  CRITICAL_SECTION csRDDOB;
  CRITICAL_SECTION csSD;

public:
  CCANDeviceEx();
  virtual ~CCANDeviceEx();

  //Standart functions 
  BOOL   ReadBar(UINT32 nAddr, void *pData, UINT32 nSize);
  BOOL   ReadReg(UINT32 nAddr, UINT32 *pData);
  BOOL   WriteBar(UINT32 nAddr, const void *pData, UINT32 nSize);
  BOOL   WriteReg(UINT32 nAddr, UINT32 pData);

  //Configuration functions
  UINT32 WriteCanReg(UINT32 nChannel, UINT32 nAddress, UINT32 nSize, void *pDataBuf);
  UINT32 ReadCanReg(UINT32 nChannel, UINT32 nAddress, UINT32 nSize, void *pDataBuf); 
  UINT32 ModifyBit(UINT32 nChannel, UINT32 nAddress, UINT16 nData);
  UINT32 EnableDMA(UINT32 nCh);
  UINT32 DisableDMA(UINT32 nCh);
  UINT32 ResetChannel(UINT32 nChannel);
  UINT32 SetMode(UINT32 nChannel, UINT32 nMode); 
  UINT32 GetMode(UINT32 nChannel, UINT32 *nMode);
  UINT32 SetConfigurationMode(UINT32 nChannel);
  UINT32 SetLastModeBeforeConf(UINT32 nChannel);
  UINT32 ReadErrors(UINT32 nChannel, UINT8 *TEC_Value, UINT8 *REC_Value, UINT8 *EFLG_Value);
  UINT32 SetSpeed(UINT32 nChannel, UINT32 SpeedValue);
  UINT32 SetSpeedByParametrs(UINT32 nChannel, UINT8 BRP, UINT8 SJW, UINT8 SAM, UINT8 BTLMODE,
	                         UINT8 PHSEG1, UINT8 PHSEG2, UINT8 PRSEG, UINT8 WAKFIL);
  UINT32 SetABAT(UINT32 nChannel, BOOL bFlag);
  UINT32 SetReceiveMode(UINT32 nCh, UINT32 nBuf, UINT32 nMode);
  UINT32 SetRXFn(UINT32 nCh, UINT32 nFilterNum, UINT32 nSID, UINT32 nFlag, UINT32 nEID_or_nData2Bytes);
  UINT32 SetRXMn(UINT32 nCh, UINT32 nMaskNum, UINT32 nSID, UINT32 nFlag, UINT32 nEID_or_nData2Bytes);

  //Functions for work with data
  UINT32 SendData(UINT32 nChannel, UINT32 nBufNumber, UINT32 nPriority,
				  UINT32 SID, UINT32 EID, void *pData, UINT32 nSize);
  void   DataAnalysis(const void *pSourseData, UINT32 *nBuf, UINT32 *nCh, UINT32 *nTimer,
	                  UINT32 *nSID, UINT32 *nEID, void *OnlyData, UINT32 *nSize);
  UINT32 CheckTransmit(UINT32 nCh, UINT32 nBufNumber);
  UINT32 EndTransmit(UINT32 nCh, UINT32 nBufNumber);
  UINT32 ReadDMADataOneBlock(void *pData);
  BOOL   ReadDMADataBlocks(void *pData, UINT32 nBlocks, UINT32 *pBlocks);
  BOOL   GetNBlockRawDMA(UINT32 *nValue);
  BOOL   GetDMAReadyStatus(UINT32 *bFlag);
  BOOL   GetProgCount(UINT32 *nCount);
  BOOL   ClearDMA();

  //Canonical
  UINT32 ReadDMACanonicalDataBlocks(void *pData, UINT32 nCh, UINT32 nBlocks, UINT32 *pBlocks);
  BOOL   DisableCanonicalDMA();
  BOOL   EnableCanonicalDMA();

  //Interrupts functions
  BOOL   GetIntFlagsAll(UINT32 *pIntFlags, UINT32 *pCan1IntFlags, UINT32 *pCAN2IntFlags);
  UINT32 EnableFlashInt();
  UINT32 DisableFlashInt();
  UINT32 EnableCanGlobInt(UINT32 nChannel);
  UINT32 DisableCanGlobInt(UINT32 nChannel);
  UINT32 AllCanInt(UINT32 nChannel, UINT8 Data);
  UINT32 EnableSingleCanInt(UINT32 nChannel, UINT32 BitNumber);
  UINT32 DisableSingleCanInt(UINT32 nChannel, UINT32 BitNumber);

protected:
  BOOL    m_bExchangeStarted;
  BOOL    m_bBufferedIO;
  BOOL    m_bDelayedDownlink;

};

#endif
