// testCan.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "windows.h"
#include <iostream>
#include "CANDeviceEx.h"

using namespace std;

bool testGetDMASize(){
	CCANDevice obj;
	UINT32 nSize;
	obj.Open(0);
	if (obj.GetDatabufSize(&nSize))
		cout<<"DMA size = "<<nSize<<"\n";
	else
		cout<<"I/O error";
	obj.Close();
return true;
}

bool testGetDmaAddress(){
	CCANDevice obj;
	UINT32 nAddress;
	obj.Open(0);
	if (obj.GetDatabufPhysAddr(&nAddress))
		cout<<"DMA address = "<<nAddress<<"\n";
	else
		cout<<"I/O error";
	obj.Close();
return true;
}

bool testGetBarAddress(){
	CCANDevice obj;
	UINT32 nAddress;
	obj.Open(0);
	if (obj.GetBarPhysAddr(&nAddress))
		cout<<"Bar address = "<<nAddress<<"\n";
	else
		cout<<"I/O error";
	obj.Close();
return true;
}

bool testGetPCILoc(){
	CCANDevice obj;
	UINT32 nBus, nSlot;
	obj.Open(0);
	if (obj.GetPCILocation(&nBus, &nSlot))
		cout<<"Bus number = "<<nBus<<" Slot number = "<<nSlot<<"\n";
	else
		cout<<"I/O error";
	obj.Close();
return true;
}

bool testGetRevision(){
	CCANDeviceEx objEx;
	UINT32 Rev;
	objEx.Open(0);
	if (objEx.GetRevision(&Rev))
		cout<<"Rev = "<<Rev<<"\n";
	else
		cout<<"I/O error";
	objEx.Close();
return true;
}

bool testGetDriverCompileDate(){
  CCANDeviceEx objEx;
  string sStr;
  objEx.Open(0);
  if (objEx.GetDriverCompileDate(&sStr))
   cout<<"sStr = "<<sStr<<"\n";
  else
    cout<<"I/O error";
  objEx.Close();
  return true;
}

bool testWriteBar(){
  CCANDeviceEx objEx;
  UINT32 nAddr = 0x0, nData[2];
  nData[0] = 0x11223344;
  nData[1] = 0x55667788;
  objEx.Open(0);
  if (objEx.WriteBar(nAddr, &nData, 8))
    cout<<"Data was written to 0x0 address\n";
  objEx.Close();
return true;
}

bool testReadBar(){
  CCANDeviceEx objEx;
  UINT32 pData[2], nAddr = 0x0;
  objEx.Open(0);
  if (objEx.ReadBar(nAddr, &pData, 8))
    for (int i=0; i<2; i++)
		cout<<hex<<"Data = 0x"<<pData[i]<<"\n";
  else
    cout<<"I/O error";
  objEx.Close();
  return true;
}

bool testWriteReg(UINT32 nAddr, UINT32 nData){
	CCANDeviceEx objEx;
	objEx.Open(0);
	if (objEx.WriteReg(nAddr, nData))
		cout<<hex<<"Value "<<nData<<" was written to "<<nAddr<<" reg\n";
	else
		cout<<"I/O error";
	objEx.Close();
return true;
}

bool testReadReg(UINT32 nAddr){
  CCANDeviceEx objEx;
  UINT32 pData;
  objEx.Open(0);
  if (objEx.ReadReg(nAddr, &pData))
    cout<<hex<<"From 0x"<<nAddr<<" reg was read "<<pData<<"\n";
  else
    cout<<"I/O error\n";
  objEx.Close();
  return true;
}

bool testEnableDMA(UINT32 nCh){
	CCANDeviceEx objEx;
	UINT32 nRes;
	objEx.Open(0);
	nRes = objEx.EnableDMA(nCh);
	if (nRes == 1)
		  cout<<"DMA was enable\n";
    else
		cout<<"I/O error. Code = "<<nRes<<"\n";

	objEx.Close();
return true;
}

bool testDisbleDMA(UINT32 nCh){
	CCANDeviceEx objEx;
	UINT32 nRes;
	objEx.Open(0);
	nRes = objEx.DisableDMA(nCh);
	if (nRes == 1)
		  cout<<"DMA was disable\n";
    else
		cout<<"I/O error. Code = "<<nRes<<"\n";

	objEx.Close();
return true;
}

bool testDisbleCanonical(){
	CCANDeviceEx objEx;
	objEx.Open(0);
    if (objEx.DisableCanonicalDMA())
	  cout<<"Canonical was disable\n";
    else
	  cout<<"I/O error.\n";

	objEx.Close();
return true;
}

bool testResetChannel(UINT32 nCh){
	CCANDeviceEx objEx;
	UINT32 nRes;
	objEx.Open(0);
	nRes = objEx.ResetChannel(nCh);
	switch (nRes){
	case 1:
		cout<<"Channel 1 was reset.\n";
		return true;
	case 2:
		cout<<"Invalid nCh input value.\n";
		break;
	case 3:
		cout<<"I/O error";
		break;
	default:
		cout<<"Unknown error code\n";
	}
	objEx.Close();

return false;
}

bool testSetMode(UINT32 nChannel, UINT16 nMode){
	CCANDeviceEx objEx;
	UINT32 nRes;
	objEx.Open(0);
	nRes = objEx.SetMode(nChannel, nMode);
	switch (nRes){
		case 1:
		  cout<<"Mode was set successful\n";
		  break;
		case 2:
		  cout<<"Wrong nChannel value\n";
		  break;
		case 3:
		  cout<<"Wrong nMode value\n";
		  break;
		case 4:
		  cout<<"I/O error ModifyBit\n";
		  break;
		case 5:
		  cout<<"I/O error ReadCanReg\n";
		  break;
		case 6:
		  cout<<"Timeout of changing workmode has gone\n";
		  break;

		default:
			cout<<"Unknown error code = "<<nRes<<"\n";
	}

	objEx.Close();
return true;
}

bool testGetMode(UINT32 nCh){
	CCANDeviceEx objEx;
	UINT32 nMode = 0, nErr;
	objEx.Open(0);
	nErr = objEx.GetMode(nCh, &nMode);
	if (nErr == 1)
		cout<<"Work Mode on "<<nCh<<" ch = "<<nMode<<"\n";
	else
		cout<<"I/O error. Code = "<<nErr<<endl;
	objEx.Close();
return true;
}

bool testSetConfMode(UINT32 nCh){
	CCANDeviceEx objEx;
	UINT32 nErr;
	objEx.Open(0);
	nErr = objEx.SetConfigurationMode(nCh);
	if (nErr == 1)
		cout<<"Conf Work Mode on "<<nCh<<" ch was set\n";
	else
		cout<<"I/O error. Code = "<<nErr<<endl;
	objEx.Close();
return true;
}

bool testSetLastModeBC(UINT32 nCh){
	CCANDeviceEx objEx;
	UINT32 nErr;
	objEx.Open(0);
	nErr = objEx.SetLastModeBeforeConf(nCh);
	if (nErr == 1)
		cout<<"Conf Work Mode on "<<nCh<<" ch was unset\n";
	else
		cout<<"I/O error. Code = "<<nErr<<endl;
	objEx.Close();
return true;
}

bool testReadErrors(){
	CCANDeviceEx objEx;
	UINT32 nRes, nTemp1, nTemp2, nTemp3;
	UINT8 Tec, Rec, EFLG;
	objEx.Open(0);
	nRes = objEx.ReadErrors(1, &Tec, &Rec, &EFLG);
	switch (nRes){
	case 1:
		nTemp1 = Tec; nTemp2 = Rec; nTemp3 = EFLG;
		cout<<hex<<"Errors:\nTec = 0x"<<nTemp1<<" Rec = 0x"<<nTemp2<<" EFLG = 0x"<<nTemp3<<endl;
		return true;
	case 2:
		cout<<"Invalid nCh input value.\n";
		break;
	case 3:
    case 4:
	case 5:
		cout<<"I/O error";
		break;
	default:
		cout<<"Unknown error code\n";
	}
	objEx.Close();

return false;
}

bool testSetSpeed(){
	CCANDeviceEx objEx;
	objEx.Open(0);
	UINT32 nErr = 0;

   testSetConfMode(1);
   testSetConfMode(2);

   testGetMode(1);
   testGetMode(2);

    nErr = 0;
	nErr = objEx.SetSpeedByParametrs(1, 5, 1, 0, 1, 7, 6, 2, 0);
	if (nErr != 1){
		cout<<"I/O error 2 = "<<nErr<<"\n";
		return false;
	}

	nErr = 0;
	nErr = objEx.SetSpeedByParametrs(2, 5, 1, 0, 1, 7, 6, 2, 0);
	if (nErr != 1){
		cout<<"I/O error 3 = "<<nErr<<"\n";
		return false;
	}

   testSetLastModeBC(1);
   testSetLastModeBC(2);

   testGetMode(1);
   testGetMode(2);

	cout<<"Speed set successfull"<<endl;
	
	objEx.Close();
return true;
}

bool testSetSpeed2(UINT32 value){
	CCANDeviceEx objEx;
	objEx.Open(0);
	UINT32 nErr = 0;

   testSetConfMode(1);
   testSetConfMode(2);

   testGetMode(1);
   testGetMode(2);

    nErr = 0;
	nErr = objEx.SetSpeed(1, value);
	if (nErr != 1){
		cout<<"I/O error 1 = "<<nErr<<"\n";
		return false;
	}

	nErr = 0;
	nErr = objEx.SetSpeed(2, value);
	if (nErr != 1){
		cout<<"I/O error 2 = "<<nErr<<"\n";
		return false;
	}

   testSetLastModeBC(1);
   testSetLastModeBC(2);

   testGetMode(1);
   testGetMode(2);

	cout<<"Speed set successfull"<<endl;
	
	objEx.Close();
return true;
}

bool testSetABAT(){
	CCANDeviceEx objEx;
	objEx.Open(0);
	UINT32 nErr = 0;

    testSetConfMode(1);

    nErr = 0;
	nErr = objEx.SetABAT(1, true);
	if (nErr != 1){
		cout<<"I/O error 2 = "<<nErr<<"\n";
		return false;
	}

	testSetLastModeBC(1);

	cout<<"ABAT successfully set "<<endl;
	
	objEx.Close();
return true;
}

bool testReadCanReg(UINT32 nCh, UINT32 nAddr, UINT32 nSize){
	CCANDeviceEx objEx;
	objEx.Open(0);
	UINT32 pDataBuf, nErr;
    nErr = objEx.ReadCanReg(nCh, nAddr, nSize, &pDataBuf);
	if (nErr == 1)
		cout<<hex<<"From "<<nCh<<"Ch 0x"<<nAddr<<" was read "<<pDataBuf<<endl;
	else
		cout<<"I/O error. Code = "<<nErr<<endl;
	objEx.Close();
return true;
}

bool testWriteCanReg(UINT32 nCh, UINT32 nAddr, UINT8 pData, UINT32 nSize){
	CCANDeviceEx objEx;
	objEx.Open(0);

	if (objEx.WriteCanReg(nCh, nAddr, nSize, &pData))
		cout<<hex<<"To "<<nCh<<" Ch 0x"<<nAddr<<" was written "<<pData<<endl;
	else
		cout<<"I/O error W";
	objEx.Close();
return true;
}

bool testModifyBit(){
  CCANDeviceEx objEx;
  objEx.Open(0);
  UINT16 pData;
  UINT32 nRes;

  pData = 3;
  pData |= (3 << 8);

  nRes = objEx.ModifyBit(1, 0x31, pData);
  if (nRes == 1)
	 cout<<"Data has been modified\n";
  else
	  cout<<"I/O error. Code = "<<nRes<<endl;
  return true;
}

bool testSendData(UINT32 nCh){
  CCANDeviceEx objEx;
  objEx.Open(0);
  UINT32 nRes;

  UINT32 pDataNew[2];
  pDataNew[0] = 0x04030201;
  pDataNew[1] = 0x08070605;


  nRes = objEx.SendData(nCh, 0, 0, 0xAA, 0xABA, &pDataNew , 8);
 
  if (nRes == 1)
	 cout<<"Data has been send\n";
  else
	  cout<<"I/O error. Code = "<<nRes<<endl;
  return true;
}

bool testDataAnalysis(void *pSourseData){
	CCANDeviceEx objEx;
    objEx.Open(0);
	UINT32 nCh, nBuf, nTimer, nSID, nEID, nSize;
	UINT32 OnlyData[2];

	objEx.DataAnalysis(pSourseData, &nBuf, &nCh, &nTimer, &nSID, &nEID, &OnlyData, &nSize);

	cout<<dec<<"Rec Buf = "<<nBuf<<endl;
	cout<<dec<<"Rec Ch = "<<nCh<<endl;
	cout<<dec<<"Timer = "<<nTimer<<endl;
	cout<<hex<<"SID = 0x"<<nSID<<endl;
	cout<<hex<<"EID = 0x"<<nEID<<endl;
	if (nSize != 0){
      cout<<dec<<"DataSize = "<<nSize<<endl;
	  cout<<hex<<"Data word 0 = 0x"<<OnlyData[0]<<endl;
	  cout<<hex<<"Data word 1 = 0x"<<OnlyData[1]<<endl;
	}
  return true;
}

bool testCheckTr(){
  CCANDeviceEx objEx;
  UINT32 nErr;
  objEx.Open(0);
  nErr = objEx.CheckTransmit(1,0);
  if (nErr == 1)
     cout<<"Data has been send\n";
  else
     cout<<"Some errors on CheckTransmit\n";

return true;
}

bool testEndTr(){
  CCANDeviceEx objEx;
  UINT32 nErr;
  objEx.Open(0);
  nErr = objEx.EndTransmit(1,0);
  if (nErr == 1)
     cout<<"Sending was end\n";
  else
	 cout<<"I/O error:testEndTr\n";

return true;
}


bool testReadData(){
	CCANDeviceEx objEx;
	objEx.Open(0);
	UINT32 flag=0, nErr1 = 0, nErr2 = 0;
	UINT32 buf[16];

	for (int i=0; i<2000; i++){
		nErr1 = objEx.GetDMAReadyStatus(&flag);
		if (flag){
			nErr2 = objEx.ReadDMADataOneBlock(buf);
			for (int i=0; i<16; i++)
				cout<<hex<<buf[i]<<endl;
			break;
		}
	else
		cout<<"No new data\n";
	}
	cout<<"nErr1 = "<<nErr1<<" nErr2 = "<<nErr2<<endl;

	testDataAnalysis(buf);
	objEx.Close();
return true;
}

bool testReadDMADataBlocks(){
  CCANDeviceEx objEx;
  objEx.Open(0);
  UINT32 buf[32];
  UINT32 nBlocks, count = 0;
  UINT32 nSize = 2;

  if (objEx.ReadDMADataBlocks(buf, nSize, &nBlocks)){
	  cout<<"Number of blocks have been read  = "<<nBlocks<<"\n";
	  if (nBlocks > 0)
	    for (int i=0; i<32; i++)
		  cout<<hex<<buf[i]<<"\n";
  }
  else
    cout<<"I/O error\n";

  for (int i=32; i<100; i++)
	  if (buf[i] != 0x5555)
		  count++;

  cout<<"count = "<<count<<endl;

  return true;
}

bool testGetNBlockRawDMA(){
  CCANDeviceEx objEx;
  objEx.Open(0);
  UINT32 nBlocks;

  if (objEx.GetNBlockRawDMA(&nBlocks))
    cout<<"Number of blocks with new data = "<<nBlocks<<"\n";
  else
    cout<<"I/O error\n";

  return true;
}

bool testGetRrogCount(){
	CCANDeviceEx objEx;
	UINT32 nCount;
	objEx.Open(0);
	if (objEx.GetProgCount(&nCount))
		cout<<"Prog count = "<<nCount<<"\n";
	else
		cout<<"I/O error";
	objEx.Close();
return true;
}

bool testClearDMA(){
	CCANDeviceEx objEx;
	objEx.Open(0);
	if (objEx.ClearDMA())
		cout<<"DMA has been cleared successfully\n";
	else
		cout<<"I/O error\n";
	objEx.Close();
return true;
}

bool testGetIntFlags(){
	CCANDeviceEx objEx;
	UINT32 nIntFlags, CAN1, CAN2;
	objEx.Open(0);
	if (objEx.GetIntFlagsAll(&nIntFlags, &CAN1, &CAN2))
		cout<<"Main Int Flags = "<<nIntFlags<<"\nCAN1 Int Flags = "<<CAN1<<"\nCAN2 Int Flags = "<<CAN2<<"\n";
	else
		cout<<"I/O error";
	objEx.Close();
return true;
}

bool testEnableFlashInt(){
	CCANDeviceEx objEx;
	UINT32 nErr;
	objEx.Open(0);
	nErr = objEx.EnableFlashInt();
	if (nErr == 1)
		cout<<"Int Flags has been enabled\n";
	else
		cout<<"I/O error. Code = "<<nErr<<endl;
	objEx.Close();
return true;
}

bool testDisableFlashInt(){
	CCANDeviceEx objEx;
	UINT32 nErr;
	objEx.Open(0);
	nErr = objEx.DisableFlashInt();
	if (nErr == 1)
		cout<<"Int Flags has been disabled\n";
	else
		cout<<"I/O error. Code = "<<nErr<<endl;
	objEx.Close();
return true;
}

bool testEnableCanGlobInt(){
	CCANDeviceEx objEx;
	UINT32 nErr;
	objEx.Open(0);
	nErr = objEx.EnableCanGlobInt(0);
	if (nErr == 1)
		cout<<"Can Flags has been enabled\n";
	else
		cout<<"I/O error. Code = "<<nErr<<endl;
	objEx.Close();
return true;
}

bool testDisableCanGlobInt(){
	CCANDeviceEx objEx;
	UINT32 nErr;
	objEx.Open(0);
	nErr = objEx.DisableCanGlobInt(0);
	if (nErr == 1)
		cout<<"Can Flags has been disabled\n";
	else
		cout<<"I/O error. Code = "<<nErr<<endl;
	objEx.Close();
return true;
}

bool testAllCanInt(){
	CCANDeviceEx objEx;
	objEx.Open(0);
	if (objEx.AllCanInt(0, 8))
		cout<<"Can Int Flags has been enabled\n";
	else
		cout<<"I/O error"<<endl;
	objEx.Close();
return true;
}

bool testEnableSingleCanInt(){
	CCANDeviceEx objEx;
	UINT32 nErr;
	objEx.Open(0);
	nErr = objEx.EnableSingleCanInt(0, 7);
	if (nErr == 1)
		cout<<"Bit has been enabled\n";
	else
		cout<<"I/O error. Code = "<<nErr<<endl;
	objEx.Close();
return true;
}


bool testSetRXF(UINT32 nRXF){
	CCANDeviceEx objEx;
	objEx.Open(0);

	UINT32 nErr;
	nErr = objEx.SetRXFn(1, nRXF, 0xAB, 0, 0xBA);
	if (nErr == 1)
		cout<<"RXF0 set sucessful"<<endl;
	else
		cout<<"I/O error. Code = "<<nErr<<endl;

	objEx.Close();
return true;
}



bool testSetRXM(UINT32 nRXM){
	CCANDeviceEx objEx;
	objEx.Open(0);

	UINT32 nErr;
	nErr = objEx.SetRXMn(1, nRXM, 0xAB, 2, 0xBA);
	if (nErr == 1)
		cout<<"RXM0 set sucessful"<<endl;
	else
		cout<<"I/O error. Code = "<<nErr<<endl;

	objEx.Close();
return true;
}
bool testDisableSingleCanInt(){
	CCANDeviceEx objEx;
	UINT32 nErr;
	objEx.Open(0);
	nErr = objEx.DisableSingleCanInt(0, 7);
	if (nErr == 1)
		cout<<"Bit has been disabled\n";
	else
		cout<<"I/O error. Code = "<<nErr<<endl;
	objEx.Close();
return true;
}





bool testSetReceiveMode(){
	CCANDeviceEx objEx;
	UINT32 nErr;
	objEx.Open(0);
	nErr = objEx.SetReceiveMode(2, 1, 3);
	if (nErr == 1)
		cout<<"Mode has been set\n";
	else
		cout<<"I/O error. Code = "<<nErr<<endl;
	objEx.Close();
return true;
}


bool testDMAby1block(){
  CCANDeviceEx objEx;
  objEx.Open(0);
  UINT32 nRes,
	     nSize = 1,
		 nTemp,
		 nSID,
		 nEID,
		 Count,
		 DMAflag;
  BOOL bFlag;
  UINT32 buf[16];

  UINT32 pSourseData[2];
  pSourseData[0] = 0x04030201;
  pSourseData[1] = 0x08070605;

  for (int i=0; i<35000; i++){
    nRes = objEx.SendData(1, 0, 0, 0xAA, 0xABA, &pSourseData, 8);

	if (nRes != 1){
		cout<<"SendData I/O error. Code = "<<nRes<<endl;
		return false;
	}

	for (int a=0; a<10; a++){
	  nRes = objEx.CheckTransmit(1,0);
	  if (nRes == 1){
		bFlag = true;
		break;
	  }
	}
	if (!bFlag){
	  cout<<"CheckTransmit I/O error. Code = "<<nRes<<endl;
	  return false;
	}
 
	if (!objEx.GetDMAReadyStatus(&DMAflag)){
	  cout<<"GetDMAReadyStatus I/O error. "<<endl;
	  return false;
	}

	if (DMAflag){
	  nRes = objEx.ReadDMADataOneBlock(buf);
	  objEx.DataAnalysis(buf, &nTemp, &nTemp, &nTemp, &nSID, &nEID, &pSourseData, &nTemp);

	  if ((nSID != 0xAA) || (nEID != 0xABA)){
	    objEx.GetProgCount(&Count);
	    cout<<hex<<"SID I/O error.\n SID = 0x"<<nSID<<endl;
	    cout<<hex<<"EID I/O error.\n EID = 0x"<<nEID<<endl;
	    cout<<dec<<"ProgCount = "<<Count<<endl;
        for (int j=0; i<16; i++)
		  cout<<hex<<buf[i]<<endl;
	    return false;
	  }
	  cout<<i<<endl;
	}
	else
	  cout<<"No new data"<<endl;
  }
  return true;
}

bool testDMAby3blocks(){
  CCANDeviceEx objEx;
  objEx.Open(0);
  UINT32 nRes,
	     nSize = 3,
		 nTemp,
		 nSID,
		 nEID,
		 Count;
  BOOL bFlag;
  UINT32 MainBuf[48], TempBuf[16] ;

  UINT32 pSourseData[2];
  pSourseData[0] = 0x04030201;
  pSourseData[1] = 0x08070605;

  for (int i=0; i<35000; i++){
	for (int b=0; b<3; b++){
      nRes = objEx.SendData(1, 0, 0, 0xAA, 0xABA, &pSourseData, 8);
	  if (nRes != 1){
		cout<<"SendData I/O error. Code = "<<nRes<<endl;
		return false;
	  }

	  for (int a=0; a<10; a++){
	    nRes = objEx.CheckTransmit(1,0);
	    if (nRes == 1){
		  bFlag = true;
		  break;
	    }
	  }
	  if (!bFlag){
	    cout<<"CheckTransmit I/O error. Code = "<<nRes<<endl;
	    return false;
	  }
	}

	if (!objEx.ReadDMADataBlocks(MainBuf, nSize, &nTemp)){
		cout<<"ReadDMA I/O error.\n";
		return false;
	}

	for(int offset=0; offset<48; offset+=16){
      memcpy(TempBuf,
             MainBuf + offset,
             16);

	  objEx.DataAnalysis(TempBuf, &nTemp, &nTemp, &nTemp, &nSID, &nEID, &pSourseData, &nTemp);

	  if ((nSID != 0xAA) || (nEID != 0xABA)){
	    objEx.GetProgCount(&Count);
	    cout<<hex<<"SID I/O error.\n SID = 0x"<<nSID<<endl;
	    cout<<hex<<"EID I/O error.\n EID = 0x"<<nEID<<endl;
	    cout<<dec<<"ProgCount = "<<Count<<endl;
        for (int j=0; i<16; i++)
		  cout<<hex<<TempBuf[i]<<endl;
	    return false;
	  }
    }
	cout<<i<<endl;
  }
  return true;
}

bool testDMAby3blocksCh(){
  CCANDeviceEx objEx;
  objEx.Open(0);
  UINT32 nRes,
	     nSize = 3,
		 nTemp,
		 nSID,
		 nEID,
		 Count;
  BOOL bFlag;
  UINT32 MainBuf[48], TempBuf[16] ;

  UINT32 pSourseData[2];
  pSourseData[0] = 0x04030201;
  pSourseData[1] = 0x08070605;

  if (!objEx.EnableCanonicalDMA()){
	cout<<"EnableDMA I/O error."<<endl;
	return false;
  }


  for (int i=0; i<35000; i++){
	for (int b=0; b<3; b++){
      nRes = objEx.SendData(1, 0, 0, 0xAA, 0xABA, &pSourseData, 8);
	  if (nRes != 1){
		cout<<"SendData I/O error. Code = "<<nRes<<endl;
		return false;
	  }

	  for (int a=0; a<10; a++){
	    nRes = objEx.CheckTransmit(1,0);
	    if (nRes == 1){
		  bFlag = true;
		  break;
	    }
	  }
	  if (!bFlag){
	    cout<<"CheckTransmit I/O error. Code = "<<nRes<<endl;
	    return false;
	  }
	}

	nRes = objEx.ReadDMACanonicalDataBlocks(MainBuf, 2, nSize, &nTemp);
	if(nRes != 1){
		cout<<"ReadDMA I/O error.C ode = "<<nRes<<endl;
		return false;
	}

	for(int offset=0; offset<48; offset+=16){
      memcpy(TempBuf,
             MainBuf + offset,
             16);

	  objEx.DataAnalysis(TempBuf, &nTemp, &nTemp, &nTemp, &nSID, &nEID, &pSourseData, &nTemp);

	  if ((nSID != 0xAA) || (nEID != 0xABA)){
	    objEx.GetProgCount(&Count);
	    cout<<hex<<"SID I/O error.\n SID = 0x"<<nSID<<endl;
	    cout<<hex<<"EID I/O error.\n EID = 0x"<<nEID<<endl;
	    cout<<dec<<"ProgCount = "<<Count<<endl;
        for (int j=0; i<16; i++)
		  cout<<hex<<TempBuf[i]<<endl;
	    return false;
	  }
    }
	cout<<i<<endl;
  }

  if (!objEx.DisableCanonicalDMA()){
	cout<<"DisableDMA I/O error."<<endl;
	return false;
  }
  return true;
}

int _tmain(int argc, _TCHAR* argv[]){
	
	/*testSetMode(1, 0);
	testSetMode(2, 0);
	//testSetSpeed();
	testSetSpeed2(125);
	testEnableDMA(0);*/

	//testDMAby1block();
	//testDMAby3blocks();
	//testDMAby3blocksCh();

	//testCheckTr();

	//testWriteCanReg(2, 0x60, 0x60, 1);

	//testWriteCanReg(1, 0x31, 0xE01c, 2);
	//testReadCanReg(1, 0x31, 2);

	//testDisbleCanonical();
	/*testSendData(1);
	testCheckTr();
	testReadData();*/

	//testSetReceiveMode();
	
	//testSetRXM(1);

	//testGetNBlockRawDMA();
	//testReadDMADataBlocks();

	/*testGetDMASize();
	testGetDmaAddress();
	testGetBarAddress();
	testGetPCILoc();
	testGetRevision();
	testGetDriverCompileDate();*/

	/*testWriteBar();
	testReadBar();
	testWriteReg(0x0, 0x123);
	testReadReg(0x0);*/

	/*testDisbleDMA(1);
	testEnableDMA(1);*/

	//testResetChannel(1);

	//testSetMode(1, 2);
	//testGetMode(1);
	/*testSetConfMode(1);
	testSetLastModeBC(1);*/

	//testReadErrors();

	//testSetSpeed();

	//testSetABAT();

	/*testWriteCanReg(1, 0x2b, 0x0, 1);
	testReadCanReg(1, 0x2c, 1);
	testWriteCanReg(1, 0x2b, 0x80, 1);
	testReadCanReg(1, 0x2c, 1);
	testModifyBit();*/

	//testSendData(1);

	/*testReadData();
	testReadDMADataBlocks();
	testReadDMAChDataBlocks();
	testGetNBlockRawDMA();
	testGetRrogCount();
	testClearDMA();*/

	/*testGetIntFlags();
	testEnableFlashInt();
	testDisableFlashInt();
	testEnableCanGlobInt();
	testDisableCanGlobInt();
	testAllCanInt();
	testEnableSingleCanInt();
	testDisableSingleCanInt();*/

    
	cin.get(); cin.get();
	return 0;
}