
#ifndef _CAN_DEVICE_H
#define _CAN_DEVICE_H

#include <string>
#include <vector>
#include "CANApi.h"

#ifndef sprintf_s
#define sprintf_s sprintf
#endif

/////////////////////////////////////////////////////////////////////////////
// Definitions

const char CAN_DRIVER_MASKNAME[] = "\\\\.\\CAN%d";

#define CAN_BOOTFLASH_WRITE_RETRY   10
#define CAN_LIB_NUMOF_RETRIES       20
#define CAN_LIB_FNID_GETPROPERTY    0x600
#define CAN_LIB_FNID_SETPROPERTY    0x700

typedef struct _CANSIM_CHANNEL
{
  UINT32 m_nDmaMode,
         m_nDatabufSize,
         m_nOffset,
         m_nLength;
} CANSIM_CHANNEL;

/////////////////////////////////////////////////////////////////////////////
// Class CCANDevice

class CCANDevice
{
// Definitions
public:
  enum {
    RESULT_ERROR = -1,
    RESULT_PENDING = -2
  };

// Construction
public:
  CCANDevice();
  virtual ~CCANDevice();

// Open / Close
public:
  BOOL Open(int nIndex);
  BOOL IsOpen() { return m_bOpen; }
  void Close();
  void Attach(HANDLE hDevice);
  HANDLE Detach();

// Operations
public:
  BOOL GetDatabufSize(UINT32 *pBufSize);
  BOOL GetDatabufPhysAddr(UINT32 *pBufAddr);
  BOOL GetBarPhysAddr(UINT32 *pBarAddr);
  BOOL GetRevision(UINT32 *pRevision);
  BOOL GetPCILocation(UINT32 *pBus, UINT32 *pSlot);
  BOOL GetDriverCompileDate(std::string *pStr);

// Imlpementation
protected:
  BOOL ProcessDeviceIOControl(
    UINT32 dwIoControlCode,
    LPVOID lpInBuffer,
    UINT32 nInBufferSize,
    LPVOID lpOutBuffer,
    UINT32 nOutBufferSize,
    LPDWORD lpBytesReturned,
    UINT32 nTimeoutMs = INFINITE);

  BOOL m_bOpen;
  HANDLE m_hDevice;
  BOOL m_bOverlapped;
  OVERLAPPED m_ReadOverlapped;
  OVERLAPPED m_IOOverlapped;
  int m_nIndex;
  UINT32 m_nDeviceType;

  UINT8  m_simDeviceVersion;
};

#endif
